/*-------------------------------------------------------------------*/
/*-- ADO Plotter (PLOTTER.C)                                       --*/
/*-------------------------------------------------------------------*/

/*
Das Plotter-(Architektur)ADO kennt die Plotterkoordinaten
und die Weltkoordinaten, die aktuelle Plotterposition und den Status
des Geraets.
*/

/*-------------------------------------------------------------------*/
/*-- 031096 TOPM CREATE                                            --*/
/*-------------------------------------------------------------------*/
static unsigned char * NAME     = "PLOTTER.C 031096";

/*--  Betriebssystemabhaengige Befehlsformate.    --*/
/*--  Noetig, weil bei DLL-Einsatz die WINDOWS-   --*/
/*--  spezifischen Formate nicht auf der Annahme  --*/
/*--  DS==SS beruhen.                             --*/
#ifdef WINDOWS
  #include <windows.h>
  #define SPRINTF  wsprintf
  #define STRCPY   lstrcpy
  #define STRNCPY  lstrcpyn
  #define _P       (LPSTR)
#else
   #define SPRINTF  sprintf
   #define STRCPY   strcpy
   #define STRNCPY  strncpy
   #define _P
#endif

#include "QUALITY.H"
#include "ft.h"
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>   
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>

/*-------------------------------------------------------------------*/
/*-- interne Deklarationen                                         --*/
/*-------------------------------------------------------------------*/
#define PLOTTER_ADO 0x01

/*-------------------------------------------------------------------*/
/*-- Hilfskonstruktion zum Kapseln der Datenstruktur TPlotter      --*/
/*-------------------------------------------------------------------*/
/*-- Verhindert, dass in PLOTTER.H der Typ TPlotter als VOID-Pointer-*/
/*-- deklariert wird.                                              --*/
typedef struct
   {
   char          MagicByte;
   /*-- Datenstruktur zur Ansteuerung des fischertechnik-Interfaces --*/
   struct        FISCH *fischptr;
   int           X;
   int           Y;
   int           pX;
   int           pY;
   } TPlotterDS;


#define TPlotter TPlotterDS *
#include "PLOTTER.H"

#undef  TPlotter
typedef TPlotterDS * TPlotter;

/*-------------------------------------------------------------------*/
/*-- Das Plotter-Objekt                                            --*/
/*-------------------------------------------------------------------*/
TPlotter aktPlotter = NULL;
struct FISCH fisch;

/*-------------------------------------------------------------------*/
/*-- private Funktionsdeklarationen                                --*/
/*-------------------------------------------------------------------*/
TPlotter      Plotter_Create          ();
TPlotter      Plotter_Dispose         ();
void          warte                   ();
void          PlusX                   ();
void          MinusX                  ();
void          PlusY                   ();
void          MinusY                  ();
void          NullX                   ();
void          NullY                   ();
void          Plotter_GoTo            (int x, int y);

/*-------------------------------------------------------------------*/
/*-- Modul-globale Variable                                        --*/
/*-------------------------------------------------------------------*/

static unsigned char * INIT_ERR = "PlotterADO n.vorhanden";
static unsigned char MldStr [250];


/*-------------------------------------------------------------------*/
/*--                                                               --*/
/*-------------------------------------------------------------------*/
TPlotter Plotter_Create()

{
TPlotter aktPlotter = NULL;
static unsigned char * FUNKTION ="Create";

JoinProc (NAME, FUNKTION);
aktPlotter = ( TPlotter) malloc (sizeof (TPlotterDS));
if ( aktPlotter == NULL)
   FatalError (NAME, FUNKTION, "malloc");

aktPlotter->MagicByte          = PLOTTER_ADO;
aktPlotter->fischptr           = &fisch;
aktPlotter->X                  = -1;
aktPlotter->Y                  = -1;
aktPlotter->pX                 = -1;
aktPlotter->pY                 = -1;

finit (aktPlotter->fischptr);
LeaveProc ();
return aktPlotter;
}

/*-------------------------------------------------------------------*/
/*--                                                               --*/
/*-------------------------------------------------------------------*/
TPlotter  Plotter_Dispose   ( TPlotter inoutPlotter)

{
static unsigned char * FUNKTION ="Dispose";
static unsigned char tmp [10];

JoinProc (NAME, FUNKTION);

if (inoutPlotter != NULL)
   {
   if (inoutPlotter->MagicByte != PLOTTER_ADO)
      FatalError (NAME, "Dispose", INIT_ERR);

   /*-- MagicByte auf Null setzen, damit der freigegebene --*/
   /*-- Speicherbereich auch bei fehlerhaftem, d.h. un-   --*/
   /*-- initialisiertem Zugriff nicht mehr als Datei-ADO  --*/
   /*-- erkannt wird.                                     --*/
   inoutPlotter->MagicByte  = 0x00;
   free ( inoutPlotter);
   }
   
inoutPlotter = NULL;
LeaveProc ();
return inoutPlotter;
}

/*-------------------------------------------------------------------*/
/*-- Stift heben                                                   --*/
/*-------------------------------------------------------------------*/
void Plotter_PenUp ()

{
static unsigned char * FUNKTION ="PenUp";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

X4A (aktPlotter->fischptr);
LeaveProc ();
}

/*-------------------------------------------------------------------*/
/*-- Stift senken                                                  --*/
/*-------------------------------------------------------------------*/
void Plotter_PenDown ()

{
static unsigned char * FUNKTION ="PenDown";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

/*-- Motoren abschalten, sonst reicht der Strom nicht fuer den --*/
/*-- Elektromagneten.                                          --*/
finit (aktPlotter->fischptr);
X4R (aktPlotter->fischptr);
/*-- Die Mechanik braucht etwas Zeit ...   --*/
sleep (500);

LeaveProc ();
}

/*-------------------------------------------------------------------*/
/*-- Plotter auf Nullpunkt fahren und Koordinaten initialisieren   --*/
/*-------------------------------------------------------------------*/
void Plotter_Init ()

{
static unsigned char * FUNKTION ="Init";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

NullX ();
NullY ();

aktPlotter->X  = 0;
aktPlotter->Y  = 0;
aktPlotter->pX = 0;
aktPlotter->pY = 0;

LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Von der aktuellen Position zu den Koordinaten X, Y        */
/*------------------------------------------------------------*/
void Plotter_PU (int x, int y)

{
static unsigned char * FUNKTION ="PU";
JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();
if (aktPlotter->X < 0)
   FatalError (NAME, FUNKTION, "Plotter n. initialisiert");
Plotter_PenUp ();
Plotter_GoTo (x, y);

LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Von der aktuellen Position zu den Koordinaten X, Y        */
/*------------------------------------------------------------*/
void Plotter_PD (int x, int y)

{
static unsigned char * FUNKTION ="PD";
JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();
if (aktPlotter->X < 0)
   FatalError (NAME, FUNKTION, "Plotter n. initialisiert");
Plotter_PenDown ();
Plotter_GoTo (x, y);

LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Von der aktuellen Position zu den Koordinaten X, Y        */
/*------------------------------------------------------------*/
void Plotter_GoTo (int x, int y)

{
int stepsX, stepsY, steps;
int schritte;
int  oktant = 1;
int i, quer;

static unsigned char * FUNKTION ="GoTo";
JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();
if (aktPlotter->X < 0)
   FatalError (NAME, FUNKTION, "Plotter n. initialisiert");

steps  = 0;
stepsX = aktPlotter->pX - x;
stepsY = aktPlotter->pY - y;

if (abs (stepsX) > abs (stepsY))
   {
   if ( (x > aktPlotter->pX) && (y > aktPlotter->pY))
      oktant = 1;
   else
   if ( (x > aktPlotter->pX) && (y <= aktPlotter->pY))
      oktant = 2;
   else
   if ( (x <= aktPlotter->pX) && (y > aktPlotter->pY))
      oktant = 3;
   else
      oktant = 4;
   schritte = abs (stepsX);
   if (stepsY != 0)
      quer     = abs (stepsX) / abs (stepsY);
   else
      quer     = 0;
   }
else
   {
   if ( (x > aktPlotter->pX) && (y > aktPlotter->pY))
      oktant = 5;
   else
   if ( (x > aktPlotter->pX) && (y <= aktPlotter->pY))
      oktant = 6;
   else
   if ( (x <= aktPlotter->pX) && (y > aktPlotter->pY))
      oktant = 7;
   else
      oktant = 8;
   schritte = abs (stepsY);
   if (stepsX != 0)
      quer     = abs (stepsY) / abs (stepsX);
   else
      quer     = 0;
   }


fprintf (stderr, "schritte %d stepsX: %d stepsY: %d quer: %d X: %d aktX: %d Y: %d aktY: %d oktant: %d\n",
            schritte,
            stepsX,
            stepsX,
            quer,
            x,
            aktPlotter->pX,
            y,
            aktPlotter->pY,
            oktant);

switch (oktant)
   {
   case 1:
      {
      steps = 1;
      for (i = 0; i < schritte; i++)
         {
         PlusX ();
         if (steps == quer)
            {
            PlusY ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 2:
      {
      steps = 1;
      for (i = 0; i < schritte; i++)
         {
         PlusX ();
         if (steps == quer)
            {
            MinusY ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 3:
      {
      steps = 1;
      for (i = 0; i < schritte; i++)
         {
         MinusX ();
         if (steps == quer)
            {
            PlusY ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 4:
      {
      steps = 1;
      for (i = 0; i < schritte; i++)
         {
         MinusX ();
         if (steps == quer)
            {
            MinusY ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 5:
     {
     steps = 1;
      for (i = 0; i < schritte; i++)
         {
         PlusY ();
         if (steps == quer)
            {
            PlusX ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 6:
     {
     steps = 1;
      for (i = 0; i < schritte; i++)
         {
         MinusY ();
         if (steps == quer)
            {
            PlusX ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 7:
     {
     steps = 1;
      for (i = 0; i < schritte; i++)
         {
         PlusY ();
         if (steps == quer)
            {
            MinusX ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   case 8:
     {
     steps = 1;
      for (i = 0; i < schritte; i++)
         {
         MinusY ();
         if (steps == quer)
            {
            MinusX ();
            steps = 1;
            }
         else
            steps++;
         }
      break;
      }
   }
printf ("Aktuelle Koordinaten X: %d Y: %d\n", aktPlotter->pX,aktPlotter->pY);
LeaveProc ();
}


/*------------------------------------------------------------*/
/*  X-Nullstellung anfahren                                   */
/*------------------------------------------------------------*/
void NullX ()

{
int i;

static unsigned char * FUNKTION ="NullX";
JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

XDE (aktPlotter->fischptr);
while (aktPlotter->fischptr->schalter[6] == EIN)
   {
   MinusX ();
   XDE (aktPlotter->fischptr);
   }
finit (aktPlotter->fischptr);
for (i=0; i < 10; i++)
   PlusX ();
finit (aktPlotter->fischptr);

LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Y-Nullstellung anfahren                                   */
/*------------------------------------------------------------*/
void NullY (struct FISCH *fischptr)

{
int i;
static unsigned char * FUNKTION ="NullY";
JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();


XDE (aktPlotter->fischptr);
while (aktPlotter->fischptr->schalter[7] == EIN)
   {
   MinusY ();
   XDE (aktPlotter->fischptr);
   }
finit (aktPlotter->fischptr);
for (i=0; i < 10; i++)
   PlusY (aktPlotter->fischptr);
finit (aktPlotter->fischptr);
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  PlusX                                                     */
/*------------------------------------------------------------*/
void PlusX ()

{
int i;
void step_left  (struct FISCH *fischptr);

static unsigned char * FUNKTION ="PlusX";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

for (i=0; i < 4; i++)
   step_left (aktPlotter->fischptr);

aktPlotter->pX++;
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  MinusX                                                    */
/*------------------------------------------------------------*/
void MinusX ()

{
int i;
void step_right  (struct FISCH *fischptr);

static unsigned char * FUNKTION ="MinusX";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

for (i=0; i < 4; i++)
   step_right (aktPlotter->fischptr);

aktPlotter->pX--;
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  PlusY                                                     */
/*------------------------------------------------------------*/
void PlusY ()

{
int i;
void step_fwd  (struct FISCH *fischptr);

static unsigned char * FUNKTION ="PlusY";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();


for (i=0; i < 4; i++)
   step_fwd (aktPlotter->fischptr);

aktPlotter->pY++;
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  MinusY                                                    */
/*------------------------------------------------------------*/
void MinusY ()

{
int i;
void step_bwd  (struct FISCH *fischptr);

static unsigned char * FUNKTION ="MinusY";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

for (i=0; i < 4; i++)
   step_bwd (aktPlotter->fischptr);

aktPlotter->pY--;
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Ein Schrittmotorstep vorwaerts                            */
/*------------------------------------------------------------*/
void step_fwd  (struct FISCH *fischptr)
{
static short status = 0;
static unsigned char * FUNKTION ="step_fwd";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();

/*-- Den nicht benoetigten Motor in definierten Zustand bringen ...! --*/
X2L (aktPlotter->fischptr);

switch (status)
   {
   case 0:
      status = 1;
      warte ();
      X1R (aktPlotter->fischptr);
      X3R (aktPlotter->fischptr);
      break;
   case 1:
      status = 2;
      warte ();
      X1L (aktPlotter->fischptr);
      X3R (aktPlotter->fischptr);
      break;
   case 2:
      status = 3;
      warte ();
      X1L (aktPlotter->fischptr);
      X3L (aktPlotter->fischptr);
      break;
   case 3:
      status = 0;
      warte ();
      X1R (aktPlotter->fischptr);
      X3L (aktPlotter->fischptr);
      break;
   }
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Ein Schrittmotorstep zurueck                              */
/*------------------------------------------------------------*/
void step_bwd  (struct FISCH *fischptr)
{
static short status = 0;

static unsigned char * FUNKTION ="step_bwd";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();


/*-- Den nicht benoetigten Motor in definierten Zustand bringen ...! --*/
X2L (aktPlotter->fischptr);

switch (status)
   {
   case 0:
      status = 1;
      warte ();
      X3L (aktPlotter->fischptr);
      X1L (aktPlotter->fischptr);
      break;
   case 1:
      status = 2;
      warte ();
      X3R (aktPlotter->fischptr);
      X1L (aktPlotter->fischptr);
      break;
   case 2:
      status = 3;
      warte ();
      X3R (aktPlotter->fischptr);
      X1R (aktPlotter->fischptr);
      break;
   case 3:
      status = 0;
      warte ();
      X3L (aktPlotter->fischptr);
      X1R (aktPlotter->fischptr);
      break;
   }
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Ein Schrittmotorstep rechts                               */
/*------------------------------------------------------------*/
void step_right (struct FISCH *fischptr)
{
static short status = 0;

static unsigned char * FUNKTION ="step_right";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();


/*-- Den nicht benoetigten Motor in definierten Zustand bringen ...! --*/
X3L (aktPlotter->fischptr);

switch (status)
   {
   case 0:
      status = 1;
      warte ();
      X1R (aktPlotter->fischptr);
      X2R (aktPlotter->fischptr);
      break;
   case 1:
      status = 2;
      warte ();
      X1L (aktPlotter->fischptr);
      X2R (aktPlotter->fischptr);
      break;
   case 2:
      status = 3;
      warte ();
      X1L (aktPlotter->fischptr);
      X2L (aktPlotter->fischptr);
      break;
   case 3:
      status = 0;
      warte ();
      X1R (aktPlotter->fischptr);
      X2L (aktPlotter->fischptr);
      break;
   }
LeaveProc ();
}

/*------------------------------------------------------------*/
/*  Ein Schrittmotorstep links                                */
/*------------------------------------------------------------*/
void step_left (struct FISCH *fischptr)
{
static short status = 0;

static unsigned char * FUNKTION ="step_left";

JoinProc (NAME, FUNKTION);
if (aktPlotter == NULL)
   aktPlotter = Plotter_Create ();


/*-- Den nicht benoetigten Motor in definierten Zustand bringen ...! --*/
X3L (aktPlotter->fischptr);

switch (status)
   {
   case 0:
      status = 1;
      warte ();
      X2L (aktPlotter->fischptr);
      X1L (aktPlotter->fischptr);
      break;
   case 1:
      status = 2;
      warte ();
      X2R (aktPlotter->fischptr);
      X1L (aktPlotter->fischptr);
      break;
   case 2:
      status = 3;
      warte ();
      X2R (aktPlotter->fischptr);
      X1R (aktPlotter->fischptr);
      break;
   case 3:
      status = 0;
      warte ();
      X2L (aktPlotter->fischptr);
      X1R (aktPlotter->fischptr);
      break;

   }
LeaveProc ();
}

/*------------------------------------------------------------*/
/*    Warteschleife                                           */
/*------------------------------------------------------------*/
void warte ()
{
sleep (55);
}
